==> cms_content_sync-8.x-1.14.tar.gz: cms_content_sync/src/Plugin/cms_content_sync/field_handler/DefaultEntityReferenceHandler.php <==
<?php

namespace Drupal\cms_content_sync\Plugin\cms_content_sync\field_handler;

use Drupal\cms_content_sync\Entity\Flow;
use Drupal\Core\Config\Entity\ConfigEntityStorage;
use Drupal\Core\Entity\EntityInterface;
use Drupal\cms_content_sync\Entity\EntityStatus;
use Drupal\cms_content_sync\ExportIntent;
use Drupal\cms_content_sync\ImportIntent;
use Drupal\cms_content_sync\Plugin\FieldHandlerBase;
use Drupal\cms_content_sync\SyncIntent;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Providing a minimalistic implementation for any field type.
 *
 * @FieldHandler(
 *   id = "cms_content_sync_default_entity_reference_handler",
 *   label = @Translation("Default Entity Reference"),
 *   weight = 90
 * )
 *
 * @package Drupal\cms_content_sync\Plugin\cms_content_sync\field_handler
 */
class DefaultEntityReferenceHandler extends FieldHandlerBase {

  const SUPPORTED_CONFIG_ENTITY_TYPES = [
    'block',
    'view',
  ];

  /**
   * {@inheritdoc}
   */
  public static function supports($entity_type, $bundle, $field_name, FieldDefinitionInterface $field) {
    if (!in_array($field->getType(), ["entity_reference", "entity_reference_revisions"])) {
      return FALSE;
    }

    $type = $field->getSetting('target_type');
    if (in_array($type, ['user', 'brick_type', 'paragraph', 'menu_link_content'])) {
      return FALSE;
    }

    // Limitate support for supported config entity types.
    $field_storage = $field->getFieldStorageDefinition();
    $target = $field_storage->getPropertyDefinition('entity')
      ->getTargetDefinition()
      ->getEntityTypeId();
    $referenced_entity_type = \Drupal::entityTypeManager()->getStorage($target);
    if ($referenced_entity_type instanceof ConfigEntityStorage && !in_array($type, self::SUPPORTED_CONFIG_ENTITY_TYPES)) {
      return FALSE;
    }

    return TRUE;
  }

  /**
   * Don't expose option, but force export.
   *
   * @return bool
   */
  protected function forceReferencedEntityExport() {
    return FALSE;
  }

  /**
   * Don't expose option, but force export.
   *
   * @return bool
   */
  protected function forceReferencedEntityEmbedding() {
    return FALSE;
  }

  /**
   * Check if referenced entities should be embedded automatically.
   *
   * @param bool $default
   *   Whether to get the default value (TRUE) if none is set
   *   yet.
   *
   * @return bool
   */
  protected function shouldEmbedReferencedEntities($default = FALSE) {
    if ($this->forceReferencedEntityEmbedding()) {
      return TRUE;
    }

    if (isset($this->settings['handler_settings']['embed_referenced_entities'])) {
      return !!$this->settings['handler_settings']['embed_referenced_entities'];
    }

    if ($default) {
      return TRUE;
    }

    return FALSE;
  }

  /**
   * Check if referenced entties should be exported automatically.
   *
   * @param bool $default
   *   Whether to get the default value (TRUE) if none is set
   *   yet.
   *
   * @return bool
   */
  protected function shouldExportReferencedEntities($default = FALSE) {
    if ($this->forceReferencedEntityExport()) {
      return TRUE;
    }

    if (isset($this->settings['handler_settings']['export_referenced_entities'])) {
      return !!$this->settings['handler_settings']['export_referenced_entities'];
    }

    if ($default) {
      return TRUE;
    }

    return FALSE;
  }

  /**
   * @inheritdoc
   */
  public function getHandlerSettings() {
    $options = [];

    // Will be added in an upcoming release and recommended for paragraphs
    // and bricks.
    // Other entity types like media or taxonomy can use this as a performance
    // improvement as well.
    /*if(!$this->forceReferencedEntityEmbedding()) {
    $options = [
    'embed_referenced_entities' => [
    '#type' => 'checkbox',
    '#title' => 'Embed referenced entities',
    '#default_value' => $this->shouldEmbedReferencedEntities(),
    ],
    ];
    }*/

    // Do not add the option for references to config entities.
    $referenced_entity_type = \Drupal::entityTypeManager()->getStorage($this->getReferencedEntityType());
    if (!$referenced_entity_type instanceof ConfigEntityStorage) {
      if (!$this->forceReferencedEntityExport() && !$this->forceReferencedEntityEmbedding()) {
        $options = [
          'export_referenced_entities' => [
            '#type' => 'checkbox',
            '#title' => 'Export referenced entities',
            '#default_value' => $this->shouldExportReferencedEntities(TRUE),
          ],
        ];
      }
    }

    return $options;
  }

  /**
   * @inheritdoc
   */
  public function validateHandlerSettings(array &$form, FormStateInterface $form_state, $settings_key) {
    if (!$this->shouldExportReferencedEntities() && !$this->shouldEmbedReferencedEntities()) {
      return;
    }

    $reference_type = $this->getReferencedEntityType();

    foreach ($form['sync_entities'] as $key => $config) {
      // Ignore field definitions.
      if (substr_count($key, '-') != 1) {
        continue;
      }
      $values = $form_state->getValue(['sync_entities', $key]);
      // Ignore ignored configs.
      if ($values['handler'] == Flow::HANDLER_IGNORE) {
        continue;
      }
      // Ignore exports that are not automatic or referenced.
      if ($values['export'] == ExportIntent::EXPORT_MANUALLY) {
        continue;
      }

      list($entity_type_id,) = explode('-', $key);

      // Ignore configs that don't match our entity type.
      if ($entity_type_id != $reference_type) {
        continue;
      }

      // One has an export handler, so we can ignore this.
      return;
    }

    // No fitting handler was found- inform the user that he's missing some
    // configuration.
    if ($this->forceReferencedEntityExport() || $this->forceReferencedEntityEmbedding()) {
      $element = &$form['sync_entities'][$settings_key]['handler'];
    }
    else {
      $element = &$form['sync_entities'][$settings_key]['handler_settings']['export_referenced_entities'];
    }

    $form_state->setError(
      $element,
      \t(
        'You want to export referenced %label\'s automatically, but you have not defined any handler for this entity type. Please scroll to the bundles of this entity type, add a handler and set "export" to "referenced" there.',
        ['%label' => $reference_type]
      )
    );
  }

  /**
   * @return string
   */
  protected function getReferencedEntityType() {
    $reference_type = $this->fieldDefinition
      ->getFieldStorageDefinition()
      ->getPropertyDefinition('entity')
      ->getTargetDefinition()
      ->getEntityTypeId();

    return $reference_type;
  }

  /**
   * Load the entity that is either referenced or embedded by $definition.
   *
   * @param \Drupal\cms_content_sync\ImportIntent $intent
   * @param $definition
   *
   * @return \Drupal\Core\Entity\EntityInterface
   *
   * @throws \Drupal\cms_content_sync\Exception\SyncException
   */
  protected function loadReferencedEntity(ImportIntent $intent, $definition) {
    return $intent->loadEmbeddedEntity($definition);
  }

  /**
   * @inheritdoc
   */
  protected function setValues(ImportIntent $intent) {
    if ($intent->shouldMergeChanges()) {
      return FALSE;
    }
    /**
     * @var \Drupal\Core\Entity\EntityInterface $entity
     */
    $entity = $intent->getEntity();

    $data = $intent->getField($this->fieldName);

    $values = [];
    foreach ($data as $value) {
      $reference = $this->loadReferencedEntity($intent, $value);

      if ($reference) {
        $info = $intent->getEmbeddedEntityData($value);

        $attributes = $this->getFieldValuesForReference($reference, $intent);

        if (is_array($attributes)) {
          $values[] = array_merge($info, $attributes);
        }
        else {
          $values[] = $attributes;
        }
      }
    }

    $entity->set($this->fieldName, $values);

    return TRUE;
  }

  /**
   * Get the values to be set to the $entity->field_*.
   *
   * @param $reference
   * @param $intent
   *
   * @return array
   */
  protected function getFieldValuesForReference($reference, $intent) {
    if ($this->fieldDefinition->getType() == 'entity_reference_revisions') {
      $attributes = [
        'target_id' => $reference->id(),
        'target_revision_id' => $reference->getRevisionId(),
      ];
    }
    else {
      $attributes = [
        'target_id' => $reference->id(),
      ];
    }

    return $attributes;
  }

  /**
   * @inheritdoc
   */
  public function import(ImportIntent $intent) {
    $action = $intent->getAction();

    // Deletion doesn't require any action on field basis for static data.
    if ($action == SyncIntent::ACTION_DELETE) {
      return FALSE;
    }

    return $this->setValues($intent);
  }

  /**
   * @inheritdoc
   */
  public function export(ExportIntent $intent) {
    $action = $intent->getAction();
    /**
     * @var \Drupal\Core\Entity\EntityInterface $entity
     */
    $entity = $intent->getEntity();

    // Deletion doesn't require any action on field basis for static data.
    if ($action == SyncIntent::ACTION_DELETE) {
      return FALSE;
    }

    $data = $entity->get($this->fieldName)->getValue();

    $result = [];

    foreach ($data as $delta => $value) {
      $reference = $this->loadReferencedEntityFromFieldValue($value);

      if (!$reference || $reference->uuid() == $intent->getUuid()) {
        continue;
      }

      unset($value['target_id']);

      $result[] = $this->serializeReference($intent, $reference, $value);
    }

    $intent->setField($this->fieldName, $result);

    return TRUE;
  }

  /**
   * Load the referenced entity, given the $entity->field_* value.
   *
   * @param $value
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected function loadReferencedEntityFromFieldValue($value) {
    if (empty($value['target_id'])) {
      return NULL;
    }

    $entityTypeManager = \Drupal::entityTypeManager();
    $reference_type = $this->getReferencedEntityType();
    $storage = $entityTypeManager
      ->getStorage($reference_type);

    $target_id = $value['target_id'];
    $reference = $storage
      ->load($target_id);

    return $reference;
  }

  /**
   * @param \Drupal\cms_content_sync\ExportIntent $intent
   * @param \Drupal\Core\Entity\EntityInterface $reference
   * @param $value
   *
   * @return array
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \Drupal\cms_content_sync\Exception\SyncException
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  protected function serializeReference(ExportIntent $intent, EntityInterface $reference, $value) {
    foreach ($this->getInvalidExportSubfields() as $field) {
      unset($value[$field]);
    }
    foreach ($value as $key => $data) {
      if (substr($key, 0, 6) == 'field_') {
        unset($value[$key]);
      }
    }

    if ($this->shouldEmbedReferencedEntities()) {
      return $intent->embedEntity($reference, SyncIntent::ENTITY_REFERENCE_EMBED, $value);
    }
    elseif ($this->shouldExportReferencedEntities()) {
      return $intent->embedEntity($reference, SyncIntent::ENTITY_REFERENCE_EXPORT_AS_DEPENDENCY, $value);
    }
    else {
      return $intent->embedEntityDefinition(
        $reference->getEntityTypeId(),
        $reference->bundle(),
        $reference->uuid(),
        SyncIntent::ENTITY_REFERENCE_RESOLVE_IF_EXISTS,
        $value
      );
    }
  }

  /**
   * Get a list of array keys from $entity->field_* values that should be
   * ignored (unset before export).
   *
   * @return array
   */
  protected function getInvalidExportSubfields() {
    return ['_accessCacheability', '_attributes', '_loaded', 'top', 'target_revision_id', 'subform'];
  }

  /**
   * Save the export settings the user selected for paragraphs.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   * @param null $parent_entity
   * @param array $tree_position
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public static function saveEmbeddedExportPools(EntityInterface $entity, $parent_entity = NULL, $tree_position = []) {
    if (!$parent_entity) {
      $parent_entity = $entity;
    }
    // Make sure paragraph export settings are saved as well..
    $entityTypeManager = \Drupal::entityTypeManager();
    $entityFieldManager = \Drupal::service('entity_field.manager');
    /** @var \Drupal\Core\Field\FieldDefinitionInterface[] $fields */
    $fields = $entityFieldManager->getFieldDefinitions($entity->getEntityTypeId(), $entity->bundle());
    foreach ($fields as $name => $definition) {
      if ($definition->getType() == 'entity_reference_revisions') {
        $reference_type = $definition
          ->getFieldStorageDefinition()
          ->getPropertyDefinition('entity')
          ->getTargetDefinition()
          ->getEntityTypeId();
        $storage = $entityTypeManager
          ->getStorage($reference_type);

        $data = $entity->get($name)->getValue();
        foreach ($data as $delta => $value) {
          if (empty($value['target_id'])) {
            continue;
          }

          $target_id = $value['target_id'];
          $reference = $storage
            ->load($target_id);

          if (!$reference) {
            continue;
          }

          // In case the values are still present, favor those.
          if (!empty($value['subform']['cms_content_sync_export_group'])) {
            $set = $value['subform']['cms_content_sync_export_group'];
            EntityStatus::accessTemporaryExportPoolInfoForField($entity->getEntityTypeId(), $entity->uuid(), $name, $delta, $tree_position, $set['cms_content_sync_flow'], $set['cms_content_sync_pool'], !empty($set['cms_content_sync_uuid']) ? $set['cms_content_sync_uuid'] : NULL);
          }

          EntityStatus::saveSelectedExportPoolInfoForField($parent_entity, $name, $delta, $reference, $tree_position);

          self::saveEmbeddedExportPools($reference, $parent_entity, array_merge($tree_position, [$name, $delta, 'subform']));
        }
      }
    }
  }

}

==> cms_content_sync-8.x-1.16.tar.gz: cms_content_sync/src/Plugin/cms_content_sync/field_handler/DefaultEntityReferenceHandler.php <==
<?php

namespace Drupal\cms_content_sync\Plugin\cms_content_sync\field_handler;

use Drupal\cms_content_sync\Entity\Flow;
use Drupal\Core\Config\Entity\ConfigEntityStorage;
use Drupal\Core\Entity\EntityInterface;
use Drupal\cms_content_sync\Entity\EntityStatus;
use Drupal\cms_content_sync\ExportIntent;
use Drupal\cms_content_sync\ImportIntent;
use Drupal\cms_content_sync\Plugin\FieldHandlerBase;
use Drupal\cms_content_sync\SyncIntent;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Providing a minimalistic implementation for any field type.
 *
 * @FieldHandler(
 *   id = "cms_content_sync_default_entity_reference_handler",
 *   label = @Translation("Default Entity Reference"),
 *   weight = 90
 * )
 *
 * @package Drupal\cms_content_sync\Plugin\cms_content_sync\field_handler
 */
class DefaultEntityReferenceHandler extends FieldHandlerBase {

  const SUPPORTED_CONFIG_ENTITY_TYPES = [
    'block',
    'view',
  ];

  /**
   * {@inheritdoc}
   */
  public static function supports($entity_type, $bundle, $field_name, FieldDefinitionInterface $field) {
    if (!in_array($field->getType(), ["entity_reference", "entity_reference_revisions"])) {
      return FALSE;
    }

    $type = $field->getSetting('target_type');
    if (in_array($type, ['user', 'brick_type', 'paragraph', 'menu_link_content'])) {
      return FALSE;
    }

    // Limitate support for supported config entity types.
    $field_storage = $field->getFieldStorageDefinition();
    $target = $field_storage->getPropertyDefinition('entity')
      ->getTargetDefinition()
      ->getEntityTypeId();
    $referenced_entity_type = \Drupal::entityTypeManager()->getStorage($target);
    if ($referenced_entity_type instanceof ConfigEntityStorage && !in_array($type, self::SUPPORTED_CONFIG_ENTITY_TYPES)) {
      return FALSE;
    }

    return TRUE;
  }

  /**
   * Don't expose option, but force export.
   *
   * @return bool
   */
  protected function forceReferencedEntityExport() {
    return FALSE;
  }

  /**
   * Don't expose option, but force export.
   *
   * @return bool
   */
  protected function forceReferencedEntityEmbedding() {
    return FALSE;
  }

  /**
   * Check if referenced entities should be embedded automatically.
   *
   * @param bool $default
   *   Whether to get the default value (TRUE) if none is set
   *   yet.
   *
   * @return bool
   */
  protected function shouldEmbedReferencedEntities($default = FALSE) {
    if ($this->forceReferencedEntityEmbedding()) {
      return TRUE;
    }

    if (isset($this->settings['handler_settings']['embed_referenced_entities'])) {
      return !!$this->settings['handler_settings']['embed_referenced_entities'];
    }

    if ($default) {
      return TRUE;
    }

    return FALSE;
  }

  /**
   * Check if referenced entties should be exported automatically.
   *
   * @param bool $default
   *   Whether to get the default value (TRUE) if none is set
   *   yet.
   *
   * @return bool
   */
  protected function shouldExportReferencedEntities($default = FALSE) {
    if ($this->forceReferencedEntityExport()) {
      return TRUE;
    }

    if (isset($this->settings['handler_settings']['export_referenced_entities'])) {
      return !!$this->settings['handler_settings']['export_referenced_entities'];
    }

    if ($default) {
      return TRUE;
    }

    return FALSE;
  }

  /**
   * @inheritdoc
   */
  public function getHandlerSettings() {
    $options = [];

    // Will be added in an upcoming release and recommended for paragraphs
    // and bricks.
    // Other entity types like media or taxonomy can use this as a performance
    // improvement as well.
    /*if(!$this->forceReferencedEntityEmbedding()) {
    $options = [
    'embed_referenced_entities' => [
    '#type' => 'checkbox',
    '#title' => 'Embed referenced entities',
    '#default_value' => $this->shouldEmbedReferencedEntities(),
    ],
    ];
    }*/

    // Do not add the option for references to config entities.
    $referenced_entity_type = \Drupal::entityTypeManager()->getStorage($this->getReferencedEntityType());
    if (!$referenced_entity_type instanceof ConfigEntityStorage) {
      if (!$this->forceReferencedEntityExport() && !$this->forceReferencedEntityEmbedding()) {
        $options = [
          'export_referenced_entities' => [
            '#type' => 'checkbox',
            '#title' => 'Export referenced entities',
            '#default_value' => $this->shouldExportReferencedEntities(TRUE),
          ],
        ];
      }
    }

    return $options;
  }

  /**
   * @inheritdoc
   */
  public function validateHandlerSettings(array &$form, FormStateInterface $form_state, $settings_key) {
    if (!$this->shouldExportReferencedEntities() && !$this->shouldEmbedReferencedEntities()) {
      return;
    }

    $reference_type = $this->getReferencedEntityType();

    foreach ($form['sync_entities'] as $key => $config) {
      // Ignore field definitions.
      if (substr_count($key, '-') != 1) {
        continue;
      }
      $values = $form_state->getValue(['sync_entities', $key]);
      // Ignore ignored configs.
      if ($values['handler'] == Flow::HANDLER_IGNORE) {
        continue;
      }
      // Ignore exports that are not automatic or referenced.
      if ($values['export'] == ExportIntent::EXPORT_MANUALLY) {
        continue;
      }

      list($entity_type_id,) = explode('-', $key);

      // Ignore configs that don't match our entity type.
      if ($entity_type_id != $reference_type) {
        continue;
      }

      // One has an export handler, so we can ignore this.
      return;
    }

    // No fitting handler was found- inform the user that he's missing some
    // configuration.
    if ($this->forceReferencedEntityExport() || $this->forceReferencedEntityEmbedding()) {
      $element = &$form['sync_entities'][$settings_key]['handler'];
    }
    else {
      $element = &$form['sync_entities'][$settings_key]['handler_settings']['export_referenced_entities'];
    }

    $form_state->setError(
      $element,
      \t(
        'You want to export referenced %label\'s automatically, but you have not defined any handler for this entity type. Please scroll to the bundles of this entity type, add a handler and set "export" to "referenced" there.',
        ['%label' => $reference_type]
      )
    );
  }

  /**
   * @return string
   */
  protected function getReferencedEntityType() {
    $reference_type = $this->fieldDefinition
      ->getFieldStorageDefinition()
      ->getPropertyDefinition('entity')
      ->getTargetDefinition()
      ->getEntityTypeId();

    return $reference_type;
  }

  /**
   * Load the entity that is either referenced or embedded by $definition.
   *
   * @param \Drupal\cms_content_sync\ImportIntent $intent
   * @param $definition
   *
   * @return \Drupal\Core\Entity\EntityInterface
   *
   * @throws \Drupal\cms_content_sync\Exception\SyncException
   */
  protected function loadReferencedEntity(ImportIntent $intent, $definition) {
    return $intent->loadEmbeddedEntity($definition);
  }

  /**
   * @inheritdoc
   */
  protected function setValues(ImportIntent $intent) {
    if ($intent->shouldMergeChanges()) {
      return FALSE;
    }
    /**
     * @var \Drupal\Core\Entity\EntityInterface $entity
     */
    $entity = $intent->getEntity();

    $data = $intent->getField($this->fieldName);

    $values = [];
    foreach ($data as $value) {
      $reference = $this->loadReferencedEntity($intent, $value);

      if ($reference) {
        $info = $intent->getEmbeddedEntityData($value);

        $attributes = $this->getFieldValuesForReference($reference, $intent);

        if (is_array($attributes)) {
          $values[] = array_merge($info, $attributes);
        }
        else {
          $values[] = $attributes;
        }
      }
    }

    $entity->set($this->fieldName, $values);

    return TRUE;
  }

  /**
   * Get the values to be set to the $entity->field_*.
   *
   * @param $reference
   * @param $intent
   *
   * @return array
   */
  protected function getFieldValuesForReference($reference, $intent) {
    if ($this->fieldDefinition->getType() == 'entity_reference_revisions') {
      $attributes = [
        'target_id' => $reference->id(),
        'target_revision_id' => $reference->getRevisionId(),
      ];
    }
    else {
      $attributes = [
        'target_id' => $reference->id(),
      ];
    }

    return $attributes;
  }

  /**
   * @inheritdoc
   */
  public function import(ImportIntent $intent) {
    $action = $intent->getAction();

    // Deletion doesn't require any action on field basis for static data.
    if ($action == SyncIntent::ACTION_DELETE) {
      return FALSE;
    }

    return $this->setValues($intent);
  }

  /**
   * @inheritdoc
   */
  public function export(ExportIntent $intent) {
    $action = $intent->getAction();
    /**
     * @var \Drupal\Core\Entity\EntityInterface $entity
     */
    $entity = $intent->getEntity();

    // Deletion doesn't require any action on field basis for static data.
    if ($action == SyncIntent::ACTION_DELETE) {
      return FALSE;
    }

    $data = $entity->get($this->fieldName)->getValue();

    $result = [];

    foreach ($data as $delta => $value) {
      $reference = $this->loadReferencedEntityFromFieldValue($value);

      if (!$reference || $reference->uuid() == $intent->getUuid()) {
        continue;
      }

      unset($value['target_id']);

      $result[] = $this->serializeReference($intent, $reference, $value);
    }

    $intent->setField($this->fieldName, $result);

    return TRUE;
  }

  /**
   * Load the referenced entity, given the $entity->field_* value.
   *
   * @param $value
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected function loadReferencedEntityFromFieldValue($value) {
    if (empty($value['target_id'])) {
      return NULL;
    }

    $entityTypeManager = \Drupal::entityTypeManager();
    $reference_type = $this->getReferencedEntityType();
    $storage = $entityTypeManager
      ->getStorage($reference_type);

    $target_id = $value['target_id'];
    $reference = $storage
      ->load($target_id);

    return $reference;
  }

  /**
   * @param \Drupal\cms_content_sync\ExportIntent $intent
   * @param \Drupal\Core\Entity\EntityInterface $reference
   * @param $value
   *
   * @return array
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \Drupal\cms_content_sync\Exception\SyncException
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  protected function serializeReference(ExportIntent $intent, EntityInterface $reference, $value) {
    foreach ($this->getInvalidExportSubfields() as $field) {
      unset($value[$field]);
    }
    foreach ($value as $key => $data) {
      if (substr($key, 0, 6) == 'field_') {
        unset($value[$key]);
      }
    }

    if ($this->shouldEmbedReferencedEntities()) {
      return $intent->embedEntity($reference, SyncIntent::ENTITY_REFERENCE_EMBED, $value);
    }
    elseif ($this->shouldExportReferencedEntities()) {
      return $intent->embedEntity($reference, SyncIntent::ENTITY_REFERENCE_EXPORT_AS_DEPENDENCY, $value);
    }
    else {
      return $intent->embedEntityDefinition(
        $reference->getEntityTypeId(),
        $reference->bundle(),
        $reference->uuid(),
        SyncIntent::ENTITY_REFERENCE_RESOLVE_IF_EXISTS,
        $value
      );
    }
  }

  /**
   * Get a list of array keys from $entity->field_* values that should be
   * ignored (unset before export).
   *
   * @return array
   */
  protected function getInvalidExportSubfields() {
    return ['_accessCacheability', '_attributes', '_loaded', 'top', 'target_revision_id', 'subform'];
  }

  /**
   * Save the export settings the user selected for paragraphs.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   * @param null $parent_entity
   * @param array $tree_position
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public static function saveEmbeddedExportPools(EntityInterface $entity, $parent_entity = NULL, $tree_position = []) {
    if (!$parent_entity) {
      $parent_entity = $entity;
    }
    // Make sure paragraph export settings are saved as well..
    $entityTypeManager = \Drupal::entityTypeManager();
    $entityFieldManager = \Drupal::service('entity_field.manager');
    /** @var \Drupal\Core\Field\FieldDefinitionInterface[] $fields */
    $fields = $entityFieldManager->getFieldDefinitions($entity->getEntityTypeId(), $entity->bundle());
    foreach ($fields as $name => $definition) {
      if ($definition->getType() == 'entity_reference_revisions') {
        $reference_type = $definition
          ->getFieldStorageDefinition()
          ->getPropertyDefinition('entity')
          ->getTargetDefinition()
          ->getEntityTypeId();
        $storage = $entityTypeManager
          ->getStorage($reference_type);

        $data = $entity->get($name)->getValue();
        foreach ($data as $delta => $value) {
          if (empty($value['target_id'])) {
            continue;
          }

          $target_id = $value['target_id'];
          $reference = $storage
            ->load($target_id);

          if (!$reference) {
            continue;
          }

          // In case the values are still present, favor those.
          if (!empty($value['subform']['cms_content_sync_export_group'])) {
            $set = $value['subform']['cms_content_sync_export_group'];
            EntityStatus::accessTemporaryExportPoolInfoForField($entity->getEntityTypeId(), $entity->uuid(), $name, $delta, $tree_position, $set['cms_content_sync_flow'], $set['cms_content_sync_pool'], !empty($set['cms_content_sync_uuid']) ? $set['cms_content_sync_uuid'] : NULL);
          }

          EntityStatus::saveSelectedExportPoolInfoForField($entity, $name, $delta, $reference, $tree_position);

          self::saveEmbeddedExportPools($reference, $entity, array_merge($tree_position, [$name, $delta, 'subform']));
        }
      }
    }
  }

}
